try:
    from src.classes import AddressBook
    from src.services import *
    from src.disk import save_to_json, load_from_json
except ModuleNotFoundError:
    from .src.classes import AddressBook
    from .src.services import *
    from .src.disk import save_to_json, load_from_json
from termcolor import colored
from prompt_toolkit import PromptSession
from prompt_toolkit.completion import Completer, Completion
from prompt_toolkit.formatted_text import HTML


class FirstWordCompleter(Completer):
    def __init__(self, word_list):
        self.word_list = word_list

    def get_completions(self, document, complete_event):
        text = document.text_before_cursor
        word = text.split(" ")[0]

        if " " not in text:
            for word in self.word_list:
                if word.startswith(text):
                    yield Completion(word, start_position=-len(text))


commands = [
    'close', 'exit', 'good bye', 'hello', 'help', 'all', 'delete',
    'find-contact', 'add', 'phone', 'change', 'add-birthday', 'show-birthday',
    'change-birthday', 'delete-birthday', 'birthdays', 'add-email', 'email',
    'change-email', 'delete-email', 'add-address', 'change-address',
    'show-address', 'delete-address', 'add-note', 'edit-note', 'delete-note',
    'show-notes', 'deposit', 'withdraw', 'money', 'bank'
]


def main():
    try:
        result = load_from_json()
        if isinstance(result, tuple) and len(result) == 2:
            book, notes = result
        else:
            book = result
            notes = []
    except Exception as e:
        print(f"Failed to load data: {e}")
        book = AddressBook()
        notes = []

    print(colored("🤖 Welcome to the assistant bot!", 'black', 'on_light_yellow', attrs=['bold']))
    national_symbol()
    session = PromptSession(completer=FirstWordCompleter(commands))
    # Після визначення функції main() і перед while True:
    while True:
        user_input = session.prompt(
            HTML("<ansigreen>Enter a command ===></ansigreen> ")
        ).strip()
        parts = user_input.split(" ", 1)
        command = parts[0].lower()
        args = parts[1].split() if len(parts) > 1 else []
        if command in ["close", "exit", "good bye"]:
            save_to_json(book, notes)
            print(colored("Good bye!", "blue", "on_light_green", attrs=["bold"]))
            break
        elif command == "hello":
            print(
                colored("How can I help you?", "blue", "on_light_green", attrs=["bold"])
            )
        elif command == "help":
            print(show_commands())
        # All
        elif command == "all":
            print(show_table(book))
        elif command == "contacts":
            print(show_all(book))
        elif command == "delete":
            print(delete(args, book))


# Find  c

        elif command == "find-contact":
            print(find(args, book))

        # Phone
        elif command == "add":
            print(add_contact(args, book))
        elif command == "phone":
            print(show_phone(args, book))  #
        elif command == "change":
            print(change_contact(args, book))

        # Birthday
        elif command == "add-birthday":
            print(add_birthday(args, book))
        elif command == "show-birthday":
            print(show_birthday(args, book))
        elif command == "change-birthday":
            print(change_birthday(args, book))
        elif command == "delete-birthday":
            print(delete_birthday(args, book))
        elif command == "birthdays":
            birthdays(args, book)
        # Email
        elif command == "add-email":
            print(add_email(args, book))
        elif command == "email":
            print(show_email(args, book))
        elif command == "change-email":
            print(change_email(args, book))
        elif command == "delete-email":
            print(delete_email(args, book))

        # Address
        elif command == "add-address":
            print(add_address(args, book))
        elif command == "change-address":
            print(edit_address(args, book))
        elif command == "show-address":
            print(show_address(args, book))
        elif command == "delete-address":
            print(remove_address(args, book))
        # Note
        elif command == "add-note":
            print(new_note(notes))
        elif command == "edit-note":
            print(edit_note(notes))
        elif command == "delete-note":
            print(delete_note(notes))
        elif command == "show-notes":
            print(show_notes(notes))
            print("=" * 50)

        # Money
        elif command == "deposit":
            print(deposit(args, book))
        elif command == "withdraw":
            print(withdraw(args, book))
        elif command == "money":
            print(get_money(args, book))
        elif command == "bank":
            print(f"{bank(book)} dollars in the bank right now.")

        else:
            print('Invalid command. Enter "help" for help')


if __name__ == "__main__":
    main()




from .classes import Record, Birthday, Email, Phone, Note, Note
from .check import *
from .classes import Record
from .validate import email_is_valid
from termcolor import colored
from re import fullmatch


def input_error(func):
    """Decorator for catching typos and wrong command formats"""

    def inner(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except:
            return "Wrong format. Use 'help' for additional imformation."

    return inner


@input_error
def parse_input(user_input):
    """Splits input to command and arguments"""
    cmd, *args = user_input.split()
    cmd = cmd.strip().lower()
    return cmd, *args


# Phone -----------------------------------------------------------------------------
@input_error
def add_contact(args, book):
    """
    Function to add a new contact.

    Args:
        args: Command line arguments.
        book: Dictionary with contacts.

    Returns:
        String with information about the addition result.
    """
    name, phone = args
    if len(phone) != 10 or not phone.isdigit():
        return colored(
            "Error: Invalid phone number format. Please enter a 10-digit number.",
            "red",
            attrs=["bold"],
        )

    record = Record(name)
    record.add_phone(phone)
    book[name] = record
    return colored("✅ Contact added.", "green", attrs=["bold"])


def show_phone(args, book):
    """Shows contact`s phone number"""
    (name,) = args
    if name in book:
        record = book[name]

        res = []
        for phone in record.phones:
            res.append(phone.value)
        return f"{name}: {','.join(res)}"
    else:
        return colored(
            "Sorry, {name} doesn't exist. Use 'add' to append this contact.",
            "red",
            attrs=["bold"],
        )


@input_error
def change_contact(args, book):
    """
    Function to change an existing contact.

    Args:
        args: Command line arguments.
        book: Dictionary with contacts.

    Returns:
        String with information about the change result.
    """
    name, field, new_value = args
    if name in book:
        record = book[name]
        if field == "phone":
            record.phones = [Phone(new_value)]
        elif field == "birthday":
            record.birthday = Birthday(new_value)
        elif field == "email":
            record.emails = [Email(new_value)]
        elif field == "note":
            record.note = new_value
        return colored(f"\U0001F4DD Contact {name} updated.", "cyan", attrs=["bold"])
    else:
        return colored(f"\U000026D4 Sorry, {name} not found.", "red", attrs=["bold"])


def get_phones(record):  # Service for get phones from record
    res = []
    for phone in record.phones:
        res.append(phone.value)
    if res[0]:
        return ",".join(res)
    else:
        return colored("📵 No phone", "yellow", attrs=["bold"])


@input_error
def show_all(book):
    """Shows all information about every contact"""
    res = ["{:^60}".format("CONTACTS"), "{:-^60}".format("")]
    for name, record in book.items():
        emails = ", ".join(email.value for email in record.emails) or "No Email"
        phones = ", ".join(phone.value for phone in record.phones) or "No Phone"
        birthday = (
            record.birthday.value.strftime("%d.%m.%Y")
            if record.birthday
            else "No Birthday"
        )
        addresses = (
            "; ".join(
                f"{address.street}, {address.house_number}, {address.city}, {address.postal_code if address.postal_code else ''}, {address.country if address.country else ''}"
                for address in record.addresses
            )
            or "No Address"
        )
        money = book[name].money

        contact_info = f"👤 Name: {name}\n📞 Phone: {phones}\n📧 Email: {emails}\n🎂 Birthday: {birthday}\n🏠 Address: {addresses}\n💰 Money: {money} dollars"
        res.append(contact_info)
        res.append("{:-^60}".format(""))  # Додав розділювач між контактами
    return "\n".join(res)


@input_error
def show_table(book):
    """Shows the address book in a table format"""
    res = ["{:^125}".format("CONTACTS"), "{:-^142}".format("")]

    for name, record in book.items():
        emails = ", ".join(email.value for email in record.emails) or "No Email"
        phones = ", ".join(phone.value for phone in record.phones) or "No Phone"
        birthday = (
            record.birthday.value.strftime("%d.%m.%Y")
            if record.birthday
            else "No Birthday"
        )
        addresses = (
            "; ".join(
                f"{address.street}, {address.house_number}, {address.city}, {address.postal_code if address.postal_code else ''}, {address.country if address.country else ''}"
                for address in record.addresses
            )
            or "No Address"
        )
        money = book[name].money
        contact_info = "👤 {:<12} 📞 {:<16} 📧 {:<27} 🎂 {:<15} 🏠 {:<45} 💰 {:<6}".format(name, phones, emails, birthday, addresses, money)
        res.append(contact_info)
        res.append("{:-^142}".format(""))  # Додав розділювач між контактами
    return "\n".join(res)


# Find contact info by name ------------------------------------------------
@input_error
def find(args, book):
    """Finds a contact in the address book"""
    name = args[0]
    if name in book:
        emails = ", ".join(email.value for email in book[name].emails) or "No Email"
        phones = ", ".join(phone.value for phone in book[name].phones) or "No Phone"
        birthday = (
            book[name].birthday.value.strftime("%d.%m.%Y")
            if book[name].birthday
            else "No Birthday"
        )
        addresses = (
            "; ".join(
                f"{address.street}, {address.house_number}, {address.city}, {address.postal_code if address.postal_code else ''}, {address.country if address.country else ''}"
                for address in book[name].addresses
            )
            or "No Address"
        )
        money = book[name].money

        
        contact_info = f"👤 Name: {name}\n📞 Phone: {phones}\n📧 Email: {emails}\n🎂 Birthday: {birthday}\n🏠 Address: {addresses}\n💰 Money: {money} dollars"

        return contact_info
    else:
        return colored(f"\U000026D4 Contact {name}  not found.", "red", attrs=["bold"])


# Birthday ----------------------------------------------------------------
@input_error
def add_birthday(args, book):
    """Adds birthday to contact info"""
    name, birthday = args
    if name in book:
        record = book[name]
        record.add_birthday(birthday)
        return colored(f"🎉 {name}'s birthday added", "green", attrs=["bold"])
    else:
        return colored(
            f"❌ Sorry, {name} doesn't exist. Use 'add' to add this contact.",
            "red",
            attrs=["bold"],
        )


@input_error
def show_birthday(args, book):
    """Shows contact`s birthday"""
    (name,) = args
    if name in book:
        record = book[name]
        if record.birthday != None:
            birthday = record.birthday.value.strftime("%d.%m.%Y")
            return colored(
                f"🎂 {name}'s birthday is {birthday}", "cyan", attrs=["bold"]
            )
        else:
            return colored(
                f"❓ {name}'s birthday isn't recorded", "yellow", attrs=["bold"]
            )
    else:
        return colored(
            f"❌ Sorry, {name} doesn't exist. \nUse 'add' to add this contact to the book.",
            "red",
            attrs=["bold"],
        )


@input_error
def change_birthday(args, book):
    """Changes contact`s birthday"""
    name, birthday = args
    if name in book:
        record = book[name]
        record.add_birthday(birthday)
        return colored(f"🔄 {name}'s birthday changed", "green", attrs=["bold"])
    else:
        return colored(f"❌ Sorry, {name} doesn't exist.", "red", attrs=["bold"])


@input_error
def delete_birthday(args, book):
    """Deletes contact`s birthday"""
    (name,) = args
    if name in book:
        record = book[name]
        if record.birthday != None:
            res = record.remove_birthday()
            return res
        else:
            return colored(f"🚫 No birthday for {name}", "yellow", attrs=["bold"])
    else:
        return colored(
            f"❌ Sorry, {name} doesn't exist. \nUse 'add' to add this contact to book.",
            "red",
            attrs=["bold"],
        )


@input_error
def birthdays(args, book):
    """Returns all birthdays in a certain amount of days"""
    if args:
        days = args[0]
        book.get_birthdays_by_days(days)
    else:
        book.get_birthdays_per_week()


# Email  ----------------------------------------------------------------------
@input_error
def get_emails(record):
    """
    Function to retrieve emails from a record.

    Args:
        record: Record object.

    Returns:
        String with email addresses.
    """
    res = [email.value for email in record.emails]
    if res:
        return ",".join(res)
    else:
        return "Sorry, {name} doesn't exist. Use 'add' to append this contact."


# "add-email name email": "adding email to existing contact"
@input_error
def add_email(args, book):
    """
    Function to add an email to a contact record.

    Args:
        args (tuple): A tuple containing the name (str) and email (str) to add.
        book (dict): A dictionary representing the address book.

    Returns:
        str: A confirmation message of adding the email.
    """
    name, email = args
    if name in book:
        record = book[name]
        record.emails.append(Email(email))
        return colored(f"✅ {name}'s email added successfully", "green", attrs=["bold"])
    else:
        return colored(
            f"❌ Sorry, {name} doesn't exist. \nUse 'add' to add this contact to the book.",
            "red",
            attrs=["bold"],
        )


# "email name": "get email of specific contact"
@input_error
def show_email(args, book):
    """
    Function to get the email of a specific contact.

    Args:
        args (tuple): A tuple containing the name (str) of the contact.
        book (dict): A dictionary representing the address book.

    Returns:
        str: A string containing the contact's name and their email(s).

    """

    name = args[0]

    if name in book:
        record = book[name]
        res = []
        for email in record.emails:
            res.append(email.value)
        return f"{name}: {','.join(res)}"
    else:

        return colored(
            f"❌ Sorry, {name} doesn't exist. \nUse 'add' to add this contact to the book.",
            "red",
            attrs=["bold"],
        )


@input_error
def change_email(args, book):
    """Changes contact`s email"""
    name = args[0]
    if name in book:
        if book[name].emails:
            for number, email in enumerate(book[name].emails, 1):
                print(f"{number}: {email}")
            while True:
                answer = input("Type the number of email you want to change ===>  ")
                if answer.isdigit() and 1 <= int(answer) <= len(book[name].emails):
                    while True:
                        new_email = input("Enter new email ==>  ")
                        if email_is_valid(new_email):
                            book[name].emails[int(answer) - 1] = Email(new_email)
                            return colored(
                                "Email has been changed.", "green", attrs=["bold"]
                            )
                        else:
                            print(
                                colored(
                                    "Seems like this email is incorrect. Try again.",
                                    "red",
                                )
                            )
                else:
                    print(
                        colored(
                            f"Must be the number between 1 and {len(book[name].emails)}",
                            "red",
                        )
                    )
        else:
            return colored("Contact has no emails.", "red")
    else:
        return colored("No such contact.", "red")


# "change-email name email": "changing email of existing contact"
@input_error
def delete_email(args, book):
    """
    Function to delete an email from an existing contact.

    Args:
        args (tuple): A tuple containing the name (str) of the contact and the email (str) to delete.
        book (dict): A dictionary representing the address book.

    Returns:
        str: A confirmation message of deleting the email.

    Raises:
        ValueError: If the input arguments are not in the correct format.
    """
    name, email_to_delete = args
    if name in book:
        record = book[name]
        for email in record.emails:
            if email.value == email_to_delete:
                record.emails.remove(email)
                return colored(
                    f"🗑️ Email {email_to_delete} deleted from {name}'s contacts.",
                    "green",
                    attrs=["bold"],
                )
        return colored(
            f"🔍 Email {email_to_delete} not found in {name}'s contacts.",
            "yellow",
            attrs=["bold"],
        )
    else:
        print(
            colored(
                f"🚫 Sorry, {name} doesn't exist. Use 'add' to append this contact.",
                "red",
                attrs=["bold"],
            )
        )


# Address -----------------------------------------------------------------
@input_error
def add_address(args, book):
    """Adds contact`s address"""
    name, street, house_number, city, postal_code, country = args
    if name in book.data:
        record = book.data[name]
        record.add_address(street, house_number, city, postal_code, country)
        return colored("🏠 Address added.", "green", attrs=["bold"])
    else:
        return colored("🚫 Contact does not exist.", "red", attrs=["bold"])


@input_error
def show_address(args, book):
    """Shows contact`s address"""
    name = args[0]
    if name in book.data:
        record = book.data[name]
        addresses = record.addresses
        if addresses:
            address_str = "\n".join([f"{address}" for address in addresses])
            return colored(
                f"🏠 Addresses for {name}:\n{address_str}", "blue", attrs=["bold"]
            )
        else:
            return colored(f"❌ No addresses found for {name}.", "red")

    else:
        return colored("❌ Contact does not exist.", "red")


def edit_address(args, book):
    """Changes contact`s address"""
    name, street, house_number, city, postal_code, country = args
    if name in book.data:
        record = book.data[name]
        record.edit_address(street, house_number, city, postal_code, country)
        return colored("Address edited.", "green", attrs=["bold"])
    else:
        return colored("❌ Contact does not exist.", "red")


def remove_address(args, book):
    """Deletes contact`s address"""
    (name,) = args
    if name in book.data:
        record = book.data[name]
        record.remove_address()
        return colored("🗑️ Address removed.", "yellow", attrs=["bold"])
    else:
        return colored("❌ Contact does not exist.", "red")


# Note --------------------------------------------------------------------


def new_note(notes):
    """Adds new note with title, text and tag(s)"""
    while True:
        title = input("Type the title here ===>  ")
        if title:
            break
        else:
            print("Title cannot be empty.")
            continue
    text = input("Type the text here ===>  ")
    while True:
        tags = input(
            "Type tag(s) here (for example: <#tag1> <#multiple_word_tag_2> <#tag3>)  ===>  "
        ).split()
        if all(fullmatch(r"\#\w+", tag) for tag in tags):
            notes.append(Note(title, text, tags))
            return "Note has been added"
        else:
            print("Wrong format.")


def edit_note(notes):
    """Allows user to edit any part of his note"""
    if notes:
        note = find_note(notes)
        answer = input(
            "Type <y> if you want to change the title or any else key to continue ===>  "
        )
        if answer == "y":
            while True:
                answer = input("Type the new title here ===>  ")
                if answer:
                    note.data["title"] = input("Type the new title here ===>  ")
                else:
                    "Title cannot be empty."
        answer = input(
            "Type <y> if you want to change the text or any else key to continue ===>  "
        )
        if answer == "y":
            note.data["text"] = input("Type the new text here ===>  ")
        answer = input(
            "Type <y> if you want to change tag(s) or any else key to continue ===>  "
        )
        if answer == "y":
            while True:
                tags = input(
                    "Type tag(s) here (for example: <#tag1> <#multiple_word_tag_2> <#tag3>)  ===>  "
                ).split()
                if all(fullmatch(r"\#\w+", tag) for tag in tags):
                    note.data["tags"] = tags
                    break
                else:
                    print("Wrong format.")
        return "Note has been edited."
    return "No notes added."


def delete_note(notes):
    if notes:
        note = find_note(notes)
        del notes[notes.index(note)]
        return "Note has been deleted."
    return "No notes added."


def find_note(notes):
    """Finds a certain note in list"""
    note_dict = dict(zip(range(1, len(notes) + 1), notes))
    for number, note in note_dict.items():
        print(str(number) + ": " + note.data["title"])
    while True:
        answer = input("Type the number of note ===>  ")
        if answer.isdigit() and 1 <= int(answer) <= len(notes):
            return notes[int(answer) - 1]
        else:
            print(f"Must be the number between 1 and {len(notes)}")
            continue


def show_notes(notes):
    """Shows ceratin notes from all"""
    if notes:
        while True:
            answer = input(
                "What are we looking for? (a for all notes and s for specific) a/s ===>  "
            )
            if answer == "a":
                notes_to_print = notes
                return "\n".join(str(note) for note in notes_to_print)
            elif answer == "s":
                while True:
                    key = input(
                        "What element do you want to search by? (title/text/tags) ===>  "
                    )
                    if key in ("title", "text"):
                        element = input(f"Type the {key} you want to look for ===>  ")
                        notes_to_print = tuple(
                            filter(lambda note: element in note.data[key], notes)
                        )
                        return (
                            "\n".join(str(note) for note in notes_to_print)
                            if notes_to_print
                            else "No such notes."
                        )
                    elif key == "tags":
                        while True:
                            tags = input(
                                "Type the tag(s) you want to look for (for example: <#tag1> <#multiple_word_tag_2> <#tag3>)  ===>  "
                            ).split()
                            if all(fullmatch(r"\#\w+", tag) for tag in tags):
                                break
                            else:
                                print("Wrong format.")
                                continue
                        notes_to_print = tuple(
                            filter(
                                lambda note: all(
                                    tag in note.data["tags"] for tag in tags
                                ),
                                notes,
                            )
                        )
                        return (
                            "\n".join(str(note) for note in notes_to_print)
                            if notes_to_print
                            else "No such notes."
                        )
                    else:
                        print("Wrong format.")
                        continue
            else:
                print("Wrong format.")
                continue
    else:
        return "No notes added."


# Money -----------------------------------------------------------------------------
@input_error
def deposit(args, book):
    """Adds a specific amount of money to person`s wallet"""
    name, amount = args
    if name in book:
        if amount.isdigit() and int(amount) > 0:
            amount = int(amount)
            record = book[name]
            record.deposit(amount)
            return "Deposit successful."
        else:
            return "Invalid amount."
    else:
        return "No such contact."


@input_error
def withdraw(args, book):
    """Substracts a specific amount of money from person`s wallet"""
    name, amount = args
    if name in book:
        if amount.isdigit() and int(amount) > 0:
            amount = int(amount)
            record = book[name]
            if record.money >= amount:
                record.withdraw(amount)
                return "Withdraw successful."
            else:
                return "You cannot withdraw more than you have."
        else:
            return "Invalid amount."
    else:
        return "No such contact."


@input_error
def get_money(args, book):
    """Returns the amount of money in contact`s wallet"""
    name = args[0]
    if name in book:
        return f"{name} has {book[name].money} dollars."
    else:
        return "No such contact."


def bank(book):
    """Returns the sum of all contacts` money"""
    return sum(book[name].money for name in book)


def show_commands():
    groups = {
        "General Commands": {
            "help": "Display all available commands.",
            "exit": "Exit the program.",
        },
        "Contact Management": {
            "add [name] [phone]": "Add a new contact.",
            "delete [name]": "Delete an existing contact.",
            "change [name] [existing phone][new phone]": "Change the phone number of an existing contact.",
            "all": "Display all contacts.",
        },
        "Phone Management": {
            "phone [name]": "Display the phone number of a specific contact."
        },
        "Birthday Management": {
            "add-birthday [name] [birthday]": "Add a birthday for a contact.",
            "show-birthday [name]": "Show the birthday of a specific contact.",
            "change-birthday [name] [new birthday]": "Change birthday for your contact",
            "delete-birthday [name]": "Delete a contact's birthday.",
            "birthdays": "Displays all the birthdays for upcoming week",
            "birtdays [number of days]": "Displays all upcoming birthdays within the specified number of days."

        },
        "Email Management": {
            "add-email [name] [email]": "Add an email to a contact.",
            "email [name]": "Show the email of a specific contact.",
            "change-email [name]": "Update your contact's email.",
            "delete-email [name]": "Remove an email from a contact.",
        },
        "Address Management": {
            "add-address [name] [street] [house_number] [city] [postal_code] [country]": "Add an address to a contact.",
            "change-address [name] [new details]": "Edit an existing address.",
            "show-address [name]": "Display the address of a specific contact.",
            "delete-address [name]": "Delete an address from a contact.",
        },
        "Note Management": {
            "add-note": "Add a note to a contact.",
            "show-notes": "Display notes all or specific note.",
            "edit-note": "Edit an already existing note.",
            "delete-note": "Remove a note.",
        },
        "Money Management": {
            "deposit [name] [amount]": "deposit to person`s wallet",
            "withdraw [name] [amount]": "withdraw from person`s wallet",
            "money [name]": "show current person`s wallet balance",
            "bank": "show sum of all wallets",
        },
    }

    for group_name, commands in groups.items():
        print(f"\n{group_name}:")
        for command, description in commands.items():
            print(f"  {command:40} - {description}")
    return ""


@input_error
def delete(args, book):
    if len(args) == 1:  # if in args 1 arg
        arg = args[0]  # get arg from []
        res = book.delete(arg)
        return res
    elif len(args) == 2:
        name, field = args
        record = book.find(name)
        if field == "phones":
            res = record.remove_phones()
            return res
        elif field == "birthday":
            res = record.remove_birthday()
            return res
        elif field == "email":
            res = record.remove_email()
            return res
        elif field == "address":
            res = record.remove_address()
            return res
        elif field == "notes":
            res = record.remove_notes()
            return res


def national_symbol():
    """Glory to Ukraine"""
    print('                                                        $$             ')
    print('                                                       $$$$           ')
    print('                                           $$$         $$$$         $$$')
    print('                                           $$$$$       $$$$       $$$$$')
    print('                                           $$$ $$      $$$$      $$$$$$')
    print('                                           $$$  $$     $$$$     $$  $$$')
    print('                                           $$$   $$     $$     $$   $$$')
    print('                                           $$$   $$     $$     $$   $$$')
    print('                                           $$$   $$     $$     $$   $$$')
    print('                                           $$$   $$    $$$$    $$   $$$')
    print('                                           $$$   $$    $$$$    $$   $$$')
    print('                                           $$$  $$    $$$$$$    $$  $$$')
    print('                                           $$$$$$     $$  $$     $$$$$$')
    print('                                           $$$ $$$   $$    $$   $$$ $$$')
    print('                                           $$$  $$$$$$$    $$$$$$$  $$$')
    print('                                           $$$     $$ $$$$$$ $$     $$$')
    print('                                           $$$     $$   $$   $$     $$$')
    print('                                           $$$$$$$$$$$$$$$$$$$$$$$$$$$$')
    print('                                                   $$$  $$  $$$        ')
    print('                                                    $$$ $$ $$$         ')
    print('                                                     $$$$$$$$          ')
    print('                                                       $$$$            ')





from collections import UserDict
from datetime import datetime
from .birthdays import get_birthdays_per_week, get_birthdays_by_days
from .validate import name_is_valid, phone_is_valid, date_is_valid, email_is_valid
import re


class Field:
    """Represents a field to be filled"""

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return str(self.value)


class Name(Field):
    """Represents the contact`s name"""

    def __init__(self, value):
        if name_is_valid(value):
            self.value = value.title()
        else:
            raise NameError("Name should starts with letter")


class Phone(Field):
    """Represents the contact`s phone number"""

    def __init__(self, value):
        if phone_is_valid(value):
            self.value = value
        else:
            raise ValueError("Phone should be 10 digits format")


class Email(Field):
    """Represents the contact`s email"""

    def __init__(self, value):
        if email_is_valid(value):
            self.value = value
        else:
            raise ValueError("Invalid email format")


class Birthday(Field):
    """Represents the contact`s birthday"""

    def __init__(self, value: str):
        if date_is_valid(value):
            try:
                self.value = datetime.strptime(value, "%d.%m.%Y").date()
            except ValueError as e:
                raise ValueError("Invalid date value: " + str(e))
        else:
            raise ValueError("We couldn't validate entered date. Please, try again")


class Address(Field):
    """Represents the contact`s address"""

    def __init__(self, street, house_number, city, postal_code=None, country=None):
        self.street = street
        self.house_number = house_number
        self.city = city
        self.postal_code = postal_code
        self.country = country

    def __str__(self):
        address_parts = [self.street, self.house_number, self.city]
        if self.postal_code:
            address_parts.append(self.postal_code)
        if self.country:
            address_parts.append(self.country)
        return ", ".join(address_parts)


class Note(UserDict):
    """Represents a note with title, text and tags"""

    def __init__(self, title, text, tags):
        super().__init__()
        self.data["title"] = title
        self.data["text"] = text
        self.data["tags"] = tags

    def __str__(self):
        return f"{'=' * 50}\nTitle: {self.data['title']}\nText: {self.data['text']}\nTags: {' '.join(self.data['tags'])}"


class Record:
    """Represents a contact in your address book"""

    def __init__(self, name):
        self.name = Name(name)
        self.phones = []
        self.emails = []
        self.birthday = None
        self.addresses = []
        self.emails = []
        self.money = 0

    def add_phone(self, value):
        self.phones.append(Phone(value))

    def add_email(self, value):
        self.emails.append(Email(value))

    def remove_phone(self, phone):
        self.phones.remove(phone)

    def edit_phone(self, phone, new_phone):
        found = False
        for p in self.phones:
            if p.value == phone:
                p.value = new_phone
                print(f"Phone {phone} was changed to {new_phone}.")
                found = True
        if not found:
            raise IndexError(f"Phone {phone} wasn't found")

    def find_phone(self, phone):
        for p in self.phones:
            if p.value == phone:
                return p.value

    #
    def add_birthday(self, value):
        self.birthday = Birthday(value)
        return "birthday added"

    def __str__(self):
        if self.birthday == None:
            return f"Contact name: {self.name.value}, phones: {'; '.join(p.value for p in self.phones)}"
        else:
            return f"Contact name: {self.name.value}, phones: {'; '.join(p.value for p in self.phones)}, birthday: {self.birthday.value.strftime('%d.%m.%Y')}"

    def add_address(self, street, house_number, city, postal_code=None, country=None):
        self.addresses.append(Address(street, house_number, city, postal_code, country))

    def edit_address(self, street, house_number, city, postal_code=None, country=None):
        if self.addresses:
            address = self.addresses[0]
            if street:
                address.street = street
            if house_number:
                address.house_number = house_number
            if city:
                address.city = city
            if postal_code:
                address.postal_code = postal_code
            if country:
                address.country = country
            return "Address edited."
        else:
            return "No address to edit."

    def remove_address(self):
        self.addresses = []

    def deposit(self, amount):
        self.money += amount

    def withdraw(self, amount):
        self.money -= amount


class AddressBook(UserDict):
    """Represents an address book"""

    def __init__(self, name="contacts"):
        super().__init__()
        self.name = name

    def __dict__(self):
        return {"name": self.name, "records": dict(self.data)}  # Restructure for JSON

    def add_record(self, record):
        self.data[record.name.value] = record
        # print(f'Added new record: "{record}"')

    def find(self, value):
        for name, record in self.data.items():
            if name == value:
                return record
            else:
                raise IndexError(f"Record for {value} wasn't found")

    def delete(self, name):
        try:
            key_for_delete = None
            for key in self.data.keys():
                if key == name:
                    key_for_delete = key
            self.data.pop(key_for_delete)
            print(f"{name}'s contact was deleted")
        except KeyError:
            print(f"{name}'s contact wasn't found")

    def get_birthdays_per_week(self):
        """Shows you people who are going tot celebrate their birthday in the next seven days"""
        users = []
        for name, record in self.data.items():
            if record.birthday != None:
                users.append({"name": name, "birthday": record.birthday.value})
        return get_birthdays_per_week(users)

    def get_birthdays_by_days(self, days):
        """Shows you people who are going tot celebrate their birthday in the certain amount of days"""
        users = []
        for name, record in self.data.items():
            if record.birthday != None:
                users.append(
                    {
                        "name": name,
                        "phones": record.phones,
                        "birthday": record.birthday.value,
                    }
                )
        return get_birthdays_by_days(users, days)



import re


def is_looks_phone(string):
    """Checks if given phone is valid"""
    pattern = r"^\d{10}$"
    if re.match(pattern, string):
        return True
    else:
        return False


def is_looks_date(string):
    """Checks if given date is valid"""
    pattern = r"\d{2}\.\d{2}\.\d{4}"
    if re.match(pattern, string):
        dd, mm, yyyy = string.split(".")
        if int(dd) not in range(1, 32):
            raise ValueError("Wrong date. Day expected from 1 to 31")
        if int(mm) not in range(1, 13):
            raise ValueError("Wrong date. Month expected from 1 to 12")
        if int(yyyy) not in range(1900, 2025):
            raise ValueError("Wrong date. Year expected in range 1900-2024")
        return True
    else:
        return False
        # raise ValueError('Invalid date format. Please, use DD.MM.YYYY format')


def is_looks_email(string):
    """Checks if given email is valid"""
    pattern = r"^([a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?\.[a-zA-Z]{2,})$"
    if re.match(pattern, string):
        return True
    else:
        return False



# Import necessary modules
from datetime import datetime, timedelta
from collections import defaultdict
import os
from termcolor import colored

def get_birthdays_per_week(users):  # get users with next 7 days birthdays
    """Takes a number of days you want to look and returns all people who`s birthday is in <this number> or less days."""
    birthdays_per_week = defaultdict(list)  # For save result

    today = datetime.today().date()
    for user in users:
        name = user["name"]
        birthday = user["birthday"]
        birthday_this_year = birthday.replace(year=today.year)

        # if this year birthday pass find next year
        if birthday_this_year < today:
            birthday_this_year = birthday_this_year.replace(year=today.year + 1)

        # Find birthdays in next 7 days
        delta_days = (birthday_this_year - today).days  # days to birthdays
        if delta_days <= 7:
            day_of_week = birthday_this_year.strftime("%A")  # find day of week
            if day_of_week == "Sunday":
                day_of_week = "Monday"
            birthdays_per_week[day_of_week].append(name)  # saving result

    # Sort and print
    if len(birthdays_per_week) == 0:
        print("No users with birthday this week")

    else:
        print(colored("🎉 They have birthdays this week:", 'cyan', attrs=['bold']))
        sorted_days = sorted(birthdays_per_week.keys())
        for day in sorted_days:
            # print(day + ':  ' + ', '.join(birthdays_per_week[day]))  # simple
            print(
                "  {:<8}  {} ".format(day + ":", ", ".join(birthdays_per_week[day]))
            )  # formated
            # day from sorted_days[] but names from birthdays_per_week{}


# Function to get birthdays by days
def get_birthdays_by_days(users, string_days):
    """
    This function takes a list of users and a number of days and returns a list of users with birthdays in that number of days.

    Args:
        users (list): A list of users (dictionaries).
        days (int): The number of days from the current date.

    Returns:
        list: A list of users with birthdays in that number of days.
    """
    try:
        days = int(string_days)
    except ValueError:
        print("Error: Invalid day format. Please enter a number.")
    print("days", days)

    birthdays = []

    today = datetime.today().date()

    # Iterate through users
    for user in users:
        birthday = user["birthday"]
        birthday_this_year = birthday.replace(year=today.year)

        # Handle birthdays that have already passed this year
        if birthday_this_year < today:
            birthday_this_year = birthday_this_year.replace(year=today.year + 1)

        # Calculate delta days
        delta_days = (birthday_this_year - today).days

        # Add user to birthdays list if delta days matches input days

        if delta_days <= days:
            birthdays.append(user)

    # sorted_birthdays = sorted(birthdays, key=lambda item: item['birthday'])
    print(f"Birthdays in next {days} days:")
    for el in birthdays:
        day = el["birthday"].day
        month = el["birthday"].month
        year = el["birthday"].year
        years = today.year - year
        name = el["name"]

        phones = []
        for ph in el["phones"]:
            phone = ph.value
            phones.append(phone)

        birthday_message = f"🎉 On {day}.{month}, {name} will turn {years} years old. 📞 Phones: {', '.join(phones)}"
        print(colored(birthday_message, 'yellow', attrs=['bold']))


# Function to get birthdays by month
def get_birthdays_by_month(users, month):
    """
    This function takes a list of users and a month number and returns a list of users with birthdays in that month.

    Args:
        users (list): A list of users (dictionaries).
        month (int): The month number (1-12).

    Returns:
        list: A list of users with birthdays in that month.
    """

    birthdays = []
    # Iterate through users
    for user in users:
        birthday = user["birthday"]
        if birthday.month == month:
            birthdays.append(user)

    return birthdays



from setuptools import setup, find_namespace_packages

setup(
    name="terminaal",
    version="0.1",
    description="Address Book Assistant Bot",
    url="https://github.com/alexvekh/project-team-8-Personal-assistant",
    author="Vladyslav Babenko, Oleksy Verkhulevskyy, Aliesia Soloviova, Daniel Prokopenko, Marichka Matviiuk, Maks Pryima",
    author_email="vlad_bb@icloud.com, alexvekh@yahoo.com, aliesia.soloviova@gmail.com, megaprokop3578@gmail.com, m.v.matviiuk@gmail.com, makspryima@gmail.com",
    license="MIT",
    packages=find_namespace_packages(),
    install_requires=["termcolor", "prompt_toolkit", "setuptools"],
    entry_points={"console_scripts": ["terminaal = package.main:main"]},
)



import json
from .classes import AddressBook, Record, Name, Birthday, Phone, Email, Address, Note
from datetime import datetime

file = "package/src/data.json"


def convert_to_json(book, notes):
    """Converts given data to serializable format"""
    data = {"records": [], "notes": []}
    for record in book.data.values():
        data["records"].append(
            {
                "name": record.name.value,
                "phones": [phone.value for phone in record.phones],
                "birthday": (
                    record.birthday.value.strftime("%d.%m.%Y")
                    if record.birthday
                    else None
                ),
                "emails": [email.value for email in record.emails],
                "addresses": [str(address) for address in record.addresses],
                "money": record.money,
            }
        )
    for note in notes:
        data["notes"].append(
            {
                "title": note.data["title"],
                "text": note.data["text"],
                "tags": note.data["tags"],
            }
        )
    return data


def save_to_json(book, notes):
    """Saves data to .json file"""
    with open(file, "w", encoding="utf-8") as fh:
        json.dump(convert_to_json(book, notes), fh)
    print("Don't worry, all data saved to file.")


def load_from_json():
    """Loads data from .json file"""
    with open(file, "r") as fh:
        data = json.load(fh)
        book = AddressBook()

        # Convert to json format to book
        for contact in data["records"]:
            name = contact["name"]
            phone_list = []
            for phone in contact["phones"]:
                phone_list.append(Phone(phone))

            email_list = []
            for email in contact["emails"]:
                email_list.append(Email(email))

            addresses = [
                Address(*address_str.split(", "))
                for address_str in contact["addresses"]
            ]

            birthday = Birthday(contact["birthday"]) if contact["birthday"] else None

            money = int(contact["money"])

            record = Record(name)
            record.phones = phone_list
            record.birthday = birthday
            record.emails = email_list
            record.addresses = addresses
            record.money = money

            book.add_record(record)

        notes = [
            Note(note["title"], note["text"], note["tags"]) for note in data["notes"]
        ]
    print("Data loaded from file.")
    return book, notes
